#include <Servo.h>

// Define servo objects
Servo eyeballServo;
Servo eyelidServo;

// Define GPIO pins
const int eyeballPin = 0;
const int eyelidPin = 3;

// Define custom UART pins
#define UART_TX_PIN 16
#define UART_RX_PIN 17
#define RED 5
#define GREEN 6

// Define rotation angles
const int eyeballHome = 20;
const int eyeballRotation = 30;
const int eyelidOpen = 0;
const int eyelidClosed = 60;
const int moveDelay = 15;

// LED blinking timing
unsigned long previousMillis = 0;
const long interval = 500;  // Blink every 500ms
bool ledState = false;

void setup() {
    eyeballServo.attach(eyeballPin);
    eyelidServo.attach(eyelidPin);

    eyeballServo.write(eyeballHome);
    eyelidServo.write(eyelidOpen);

    // Initialize LEDs
    pinMode(RED, OUTPUT);
    pinMode(GREEN, OUTPUT);

    // Turn off LEDs initially
    digitalWrite(RED, HIGH);
    digitalWrite(GREEN, HIGH);

    Serial.begin(9600);  // For debugging via USB Serial
    while (!Serial) {
        // Wait for Serial Monitor to connect
    }
    Serial.println("Initializing UART...");

    // Initialize Serial1 (UART1) with custom TX and RX pins
    Serial1.setTX(UART_TX_PIN);
    Serial1.setRX(UART_RX_PIN);
    Serial1.begin(9600);  // Set baud rate for communication with ESP32
    Serial.println("UART initialized with custom TX/RX pins");
}

void loop() {
    if (Serial1.available()) {
        // Print the raw command received from ESP32 to Serial Monitor
        Serial.print("Message from ESP32: ");

        String command = Serial1.readStringUntil('\n');  // Read full command from ESP32
        command.trim();  // Remove any trailing whitespace


        Serial.println(command);

        // Process the command and execute the corresponding action
        if (command == "LEFT") {
            moveEyeball(eyeballHome - eyeballRotation);  // Turn left
            digitalWrite(GREEN, HIGH);
            delay(100);
            digitalWrite(GREEN, LOW);


        } else if (command == "RIGHT") {
            moveEyeball(eyeballHome + eyeballRotation);  // Turn right
            digitalWrite(RED, HIGH);
            delay(100);
            digitalWrite(RED, LOW);
            
        } else if (command == "BLINK") {
            blinkEyelid();  // Blink the eyelid
        } else {
            // Log unknown commands for debugging
            Serial.println("Unknown command received!");
        }
    }
}

void blinkEyelid() {
    for (int pos = eyelidOpen; pos <= eyelidClosed; pos++) {
        eyelidServo.write(pos);
        delay(moveDelay);
    }
    delay(200);
    for (int pos = eyelidClosed; pos >= eyelidOpen; pos--) {
        eyelidServo.write(pos);
        delay(moveDelay);
    }
}

void moveEyeball(int targetPosition) {
    int currentPosition = eyeballServo.read();
    int step = (currentPosition < targetPosition) ? 1 : -1;

    for (int pos = currentPosition; pos != targetPosition; pos += step) {
        eyeballServo.write(pos);
        delay(moveDelay);
    }
}
